/*	mtcpdemo.java
 *
 *	Modbus/TCP DEMO for remote I/O module
 *
 *	v 0.1 2017 Feb by moki matsushima
 *
 */

import java.util.Arrays;
import com.serotonin.modbus4j.ip.IpParameters;
import com.serotonin.modbus4j.*;
import com.serotonin.modbus4j.exception.*;
import com.serotonin.modbus4j.msg.*;
import com.serotonin.io.serial.*;

public class mtcpdemo {
	public static void main(String[] args) throws Exception {
		ModbusFactory factory = new ModbusFactory();
		IpParameters params = new IpParameters();
		//SerialParameters serialParameters = new SerialParameters();
		//serialParameters.setCommPortId("COM1");
		//serialParameters.setPortOwnerName("Numb nuts");
		//serialParameters.setBaudRate(115200);
		//serialParameters.setParity(0);
		//serialParameters.setDataBits(8);
		//serialParameters.setStopBits(1);
		params.setHost("127.0.0.1");
		params.setPort(502);
		//params.setEncapsulated(true);
		//ModbusMaster master = factory.createRtuMaster(serialParameters);
		// ModbusMaster master = modbusFactory.createAsciiMaster(serialParameters);
		ModbusMaster master = factory.createTcpMaster(params, false);
		// ModbusMaster master = modbusFactory.createUdpMaster(ipParameters);
		master.setTimeout(2000);
		master.setRetries(0);

		long start = System.currentTimeMillis();
		try {
			master.init();
			start = System.currentTimeMillis();
			readDiscreteInputTest(master,1,0,10);
		}
		finally {
			master.destroy();
		}

		System.out.println("Took: " + (System.currentTimeMillis() - start) + "ms");
	}
	public static void readCoilTest(ModbusMaster master, int slaveId, int start, int len) {
		try {
			ReadCoilsRequest request = new ReadCoilsRequest(slaveId, start, len);
			ReadCoilsResponse response = (ReadCoilsResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(Arrays.toString(response.getBooleanData()));
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void readDiscreteInputTest(ModbusMaster master, int slaveId, int start, int len) {
		try {
			ReadDiscreteInputsRequest request = new ReadDiscreteInputsRequest(slaveId, start, len);
			ReadDiscreteInputsResponse response = (ReadDiscreteInputsResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(Arrays.toString(response.getBooleanData()));
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void readHoldingRegistersTest(ModbusMaster master, int slaveId, int start, int len) {
		try {
			ReadHoldingRegistersRequest request = new ReadHoldingRegistersRequest(slaveId, start, len);
			ReadHoldingRegistersResponse response = (ReadHoldingRegistersResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(Arrays.toString(response.getShortData()));
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void readInputRegistersTest(ModbusMaster master, int slaveId, int start, int len) {
		try {
			ReadInputRegistersRequest request = new ReadInputRegistersRequest(slaveId, start, len);
			ReadInputRegistersResponse response = (ReadInputRegistersResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(Arrays.toString(response.getShortData()));
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void writeCoilTest(ModbusMaster master, int slaveId, int offset, boolean value) {
		try {
			WriteCoilRequest request = new WriteCoilRequest(slaveId, offset, value);
			WriteCoilResponse response = (WriteCoilResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println("Success");
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void writeRegisterTest(ModbusMaster master, int slaveId, int offset, int value) {
		try {
			WriteRegisterRequest request = new WriteRegisterRequest(slaveId, offset, value);
			WriteRegisterResponse response = (WriteRegisterResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println("Success");
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void readExceptionStatusTest(ModbusMaster master, int slaveId) {
		try {
			ReadExceptionStatusRequest request = new ReadExceptionStatusRequest(slaveId);
			ReadExceptionStatusResponse response = (ReadExceptionStatusResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(response.getExceptionStatus());
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void reportSlaveIdTest(ModbusMaster master, int slaveId) {
		try {
			ReportSlaveIdRequest request = new ReportSlaveIdRequest(slaveId);
			ReportSlaveIdResponse response = (ReportSlaveIdResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println(Arrays.toString(response.getData()));
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void writeCoilsTest(ModbusMaster master, int slaveId, int start, boolean[] values) {
		try {
			WriteCoilsRequest request = new WriteCoilsRequest(slaveId, start, values);
			WriteCoilsResponse response = (WriteCoilsResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println("Success");
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void writeRegistersTest(ModbusMaster master, int slaveId, int start, short[] values) {
		try {
			WriteRegistersRequest request = new WriteRegistersRequest(slaveId, start, values);
			WriteRegistersResponse response = (WriteRegistersResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println("Success");
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}

	public static void writeMaskRegisterTest(ModbusMaster master, int slaveId, int offset, int and, int or) {
		try {
			WriteMaskRegisterRequest request = new WriteMaskRegisterRequest(slaveId, offset, and, or);
			WriteMaskRegisterResponse response = (WriteMaskRegisterResponse) master.send(request);

			if (response.isException())
				System.out.println("Exception response: message=" + response.getExceptionMessage());
			else
				System.out.println("Success");
		}
		catch (ModbusTransportException e) {
			e.printStackTrace();
		}
	}
}
